 
#include <iostream>
#include <stdlib.h>     /* atoi */
#include "tcpserver.h"

class ChatServer : public AlgorithMan::TcpServer
{
    void OnConnect(AlgorithMan::Client* c) override
    {
        std::cout << "Connect " << c->RemoteIP() << std::endl;
    }
    void OnDisconnect(AlgorithMan::Client* c) override
    {
        std::cout << "Disconnect " << c->RemoteIP() << std::endl;
    }
    void OnReceive(AlgorithMan::Client* sender, const std::string& message) override
    {
        pthread_mutex_lock(&mutex);
        // forward messages to all other clients
        for(AlgorithMan::Client* client : clients)
            if(client != sender)
                Send(client, sender->RemoteIP() + ": " + message);
        pthread_mutex_unlock(&mutex);
    }
};

int main(int argc, char** argv)
{
    int port = 194;
    if(argc > 1)
        port = atoi(argv[1]);
    ChatServer server;
    server.Start(port);
    std::string input;

    try
    {
        while(true)
        {
            std::getline(std::cin, input);
            if(!std::cin.good())
                break;
        }
    }
    catch(char const* msg)
    {
        std::cerr << msg << std::endl;
    }
    
    return 0;
}
