/*
   TcpServer Class for C++ by Viktor "AlgorithMan" Engelmann
   
   This source code is provided under the terms of the
   GNU general public license version 3. See gpl3.txt for details.

   thanks to
      http://www.zotteljedi.de/
      http://www.linuxhowtos.org/C_C++/socket.htm
*/

#ifndef __ALGORITHMAN__TCPSERVER_H
    #define __ALGORITHMAN__TCPSERVER_H

    #include <iostream>
    #include <unistd.h>
    #include <fcntl.h> // open, close, read, write
    #include <stdlib.h> // atoi
    #include <pthread.h> // pthread_t etc.
    #include <arpa/inet.h>
    #include <netinet/tcp.h>
    #include <netdb.h>
    #include <string>
    #include <string.h> // memcpy
    #include <vector>
    #include <algorithm>

    namespace AlgorithMan
    {
        class TcpServer;
        
        class Client
        {
            friend void* Client_MonitorThreadProc(void* c);
            friend class TcpServer;

            private:
                TcpServer* server;

                int clientSocket;
                struct sockaddr_in client_address;
                volatile bool isConnected;
                
                pthread_t monitorThread;

                bool Accept(int listeningSocket);
            public:
                Client(TcpServer* server);
                void MonitorThreadProc();
                void Disconnect();
                void Send(const std::string& message);
                std::string RemoteIP();
        };
        
        class TcpServer
        {
            friend void* TcpServer_ListenThreadProc(void* s);
            friend class Client;
            protected:
                std::vector<Client*> clients;

                int listeningSocket;
                struct sockaddr_in server_address;
                int listeningPort;
                volatile bool isListening;
                
                pthread_t listenThread;
                pthread_mutex_t mutex;

                void NotifyDisconnect(Client* client);
            public:
                TcpServer();
                virtual ~TcpServer();
                void ListenThreadProc();
                
                void Start(int port);
                void Stop();
                virtual Client* InstantiateClient() { return new Client(this); }
                
                virtual void OnConnect(Client* client);
                virtual void OnDisconnect(Client* client);
                virtual void OnReceive(Client* sender, const std::string& message);
                void Broadcast(const std::string& message);
                void Send(Client* sender, const std::string& message);
        };
    }

#endif
